const Logger = console;
let Vue;
let observer;
const OBSERVER_OPTIONS = {
    delay: 100,
    threshold: [1],
    trackVisibility: true,
};
const elToMeta = new Map();
const $resetExposure = function (el) {
    if (el && elToMeta.has(el)) {
        const config = elToMeta.get(el);
        if (config.context === this && config.active) {
            elToMeta.set(el, Object.assign(config, { active: false }));
        }
    }
    else {
        for (let [key, config] of elToMeta.entries()) {
            if (config.context === this && config.active) {
                elToMeta.set(key, Object.assign(config, { active: false }));
            }
        }
    }
};
const createObserver = () => {
    if (window.IntersectionObserver && !observer) {
        observer = new window.IntersectionObserver((list, observer) => {
            for (let entry of list) {
                const { isIntersecting, target } = entry;
                if (isIntersecting) {
                    const config = elToMeta.get(target);
                    if (config && !config.active) {
                        const { visibility, height, width } = window.getComputedStyle(target, null);
                        if (visibility === 'hidden' ||
                            parseInt(height) === 0 ||
                            parseInt(width) === 0) {
                            break;
                        }
                        typeof config.callback === 'function' && config.callback(target);
                        elToMeta.set(target, Object.assign(config, { active: true }));
                    }
                }
            }
        }, OBSERVER_OPTIONS);
    }
    if (!observer) {
        Logger.warn('current browser does not support IntersectionObserve API');
    }
};
const addElToObserve = (el, callback, context) => {
    if (!elToMeta.has(el)) {
        elToMeta.set(el, {
            active: false,
            callback,
            context,
        });
        observer && observer.observe(el);
    }
};
const bind = (el, binding, vnode) => {
    const { value } = binding;
    const { context } = vnode;
    if (typeof value !== 'function') {
        Logger.error('directive value is not function ');
        return;
    }
    if (!context) {
        return;
    }
    if (context.$resetExposure && context.$resetExposure !== $resetExposure) {
        Logger.error('context bind $resetExposure propertyKey');
        return;
    }
    !context.$resetExposure && (context.$resetExposure = $resetExposure);
    addElToObserve(el, value, context);
};
const update = (el, binding, vnode) => {
    const { value } = binding;
    const { context } = vnode;
    if (typeof value === 'function' && elToMeta.has(el)) {
        const meta = elToMeta.get(el);
        if (!meta) {
            return;
        }
        const oldCallback = meta.callback;
        if (oldCallback !== value) {
            elToMeta.set(el, {
                active: false,
                callback: value,
                context,
            });
        }
    }
};
const unbind = (el) => {
    if (elToMeta.has(el) && observer) {
        elToMeta.delete(el);
        observer.unobserve(el);
    }
};
const installDirective = () => {
    Vue.directive('exposure', {
        bind,
        update,
        unbind,
    });
};
const install = (_Vue, options = {}) => {
    if (!Vue) {
        Vue = _Vue;
    }
    Object.assign(OBSERVER_OPTIONS, options);
    createObserver();
    installDirective();
};
const Exposure = {
    install,
};

export default Exposure;
